/*
    AppWindow.h - Mixwave main window

    Will Eighteen, 2000, 2006: will@opaline.demon.co.uk
*/

#ifndef APPWINDOW_H
#define APPWINDOW_H

#include <MenuWindow.h>
#include "OpResources.h"
#include <List.h>
#include "ChangeWaveFileCmd.h"
#include "PakDefs.h"


/* forward declarations for types */
class NoUndoCmd;
class BreakCmd;
class AsciiToFloatCmd;
class AsciiToHetroCmd;
class WavToRawCmd;
class RawToWavCmd;
class WavToIntCmd;
class IntToFloatCmd;
class IntToWavCmd;
class FloatToAsciiCmd;
class HetroToAsciiCmd;
class FloatToIntCmd;
class SplitWaveCmd;
class SysCtrlPanel;
class MixCmd;


class AppWindow : public MenuWindow
{
	public:
		AppWindow ( const char * );
		~AppWindow();

	protected:
		virtual void createMenuPanes();
		virtual Widget createWorkArea ( Widget );

	private:
		Widget _form;
		SysCtrlPanel *_sysCtrlPanel;
		Cursor _busyCursor;
		Cursor _normalCursor;
		char *currentWave;

		static void newWaveCallback ( void *, char * );
		static void loadWaveCallback ( void *, char * );
		static void fileSelectHelpCallback ( XtPointer );
		static void fileSelectCancelledCallback ( XtPointer );
		static void newSpatialTemplateCallback ( void *, char *, unsigned int );
		static void mixdownCallback ( void *, char * );
		static void copyDoneCallback ( void * );
		static void loadTemplateCallback ( void *, char *, char * );
		static void asciiToFloatCallback ( void *, char *, char * );
		static void asciiToFloatFinishedCallback ( BreakCmd *, Boolean, void * );
		static void asciiToHetroCallback ( void *, char *, char * );
		static void asciiToHetroFinishedCallback ( BreakCmd *, Boolean, void * );
		static void wavToRawCallback ( void *, char *, char * );
		static void wavToRawFinishedCallback ( BreakCmd *, Boolean, void * );
		static void rawToWavCallback ( void *, char *, char * );
		static void rawToWavFinishedCallback ( BreakCmd *, Boolean, void * );
		static void wavToIntCallback ( void *, char *, char * );
		static void wavToIntFinishedCallback ( BreakCmd *, Boolean, void * );
		static void intToFloatCallback ( void *, char *, char * );
		static void intToFloatFinishedCallback ( BreakCmd *, Boolean, void * );
		static void intToWavCallback ( void *, char *, char * );
		static void intToWavFinishedCallback ( BreakCmd *, Boolean, void * );
		static void floatToAsciiCallback ( void *, char *, char * );
		static void floatToAsciiFinishedCallback ( BreakCmd *, Boolean, void * );
		static void hetroToAsciiCallback ( void *, char *, char * );
		static void hetroToAsciiFinishedCallback ( BreakCmd *, Boolean, void * );
		static void floatToIntCallback ( void *, char *, char * );
		static void floatToIntFinishedCallback ( BreakCmd *, Boolean, void * );
		static void splitWaveCallback ( void *, char * );
		static void splitWaveFinishedCallback ( BreakCmd *, Boolean, void * );
		static void mixFinishedCallback ( BreakCmd *, Boolean, void * );
		static void textCallback ( XtPointer );
		static void textCancelledCallback ( XtPointer );

		static void splitWaveOkCallback ( void * );
		static void splitWaveCancelCallback ( void * );
		void splitWaveOk ( SplitWaveCmd * );

		void newWave ( char * );
		int loadWave ( char * );
		void loadTemplate ( char *, char * );
		void mixdown ( char * );
		void mixFinished ( MixCmd * );

		void newSpatialTemplate ( char *, unsigned int );
		void syncSelected ( char * );
		void asciiToFloat ( char *, char * );
		void asciiToFloatFinished ( AsciiToFloatCmd * );
		void asciiToHetro ( char *, char * );
		void asciiToHetroFinished ( AsciiToHetroCmd * );
		void wavToRaw ( char *, char * );
		void wavToRawFinished ( WavToRawCmd * );
		void rawToWav ( char *, char * );
		void rawToWavFinished ( RawToWavCmd * );
		void wavToInt ( char *, char * );
		void wavToIntFinished ( WavToIntCmd * );
		void intToFloat ( char *, char * );
		void intToFloatFinished ( IntToFloatCmd * );
		void intToWav ( char *, char * );
		void intToWavFinished ( IntToWavCmd * );
		void floatToAscii ( char *, char * );
		void floatToAsciiFinished ( FloatToAsciiCmd * );
		void hetroToAscii ( char *, char * );
		void hetroToAsciiFinished ( HetroToAsciiCmd * );
		void floatToInt ( char *, char * );
		void floatToIntFinished ( FloatToIntCmd * );
		void splitWave ( char * );
		void splitWaveFinished ( SplitWaveCmd * );
		
		void setBusyCursor();
		void setNormalCursor();
};

#endif
