/*
 WorkingDialogManager.cc
 modified by Will 18, 2000; will@opaline.demon.co.uk
 
 Adapted from
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/

#include "WorkingDialogManager.h"
#include "Application.h"
#include "BusyPixmapBar.h"
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <assert.h>
#include <stdlib.h>


WorkingDialogManager *workingDialogManager =
				new WorkingDialogManager ( "busy" );


WorkingDialogManager::WorkingDialogManager ( const char *name ) : DialogManager ( name )
{
	_intervalId =  0;	// orig, 'NULL'
	_busyPixmaps = NULL;
	_updateInterval = 0;
	_text[0] = 0;
}

Widget WorkingDialogManager::createDialog ( Widget parent )
{
	Widget dialog = XmCreateWorkingDialog ( parent, _name, NULL, 0 );
	XtVaSetValues ( dialog, XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, NULL );
	XtAddCallback ( dialog, XmNokCallback, &WorkingDialogManager::unpostCallback,
					( XtPointer ) this );
	XtAddCallback ( dialog, XmNcancelCallback, &WorkingDialogManager::unpostCallback,
					( XtPointer ) this );
					
	if ( !_busyPixmaps )
		_busyPixmaps = new BusyPixmap ( dialog );
						
	return ( dialog );
}


Widget WorkingDialogManager:: post ( const char *text, void *clientData,
					  DialogCallback ok, DialogCallback cancel,
					  DialogCallback help )
{
	if ( _w && XtIsManaged ( _w ) )
		return ( _w );	// dialog exists, just return it
		
	strncpy ( _text, text, sizeof ( _text ) );
	_text [ sizeof ( _text ) - 1 ] = 0;
		
	DialogManager::post ( text, clientData, ok, cancel, help );
    timer();	// start animation sequence
	return ( _w );
}


void WorkingDialogManager::timerCallback ( XtPointer clientData, XtIntervalId * )
{
	WorkingDialogManager *obj = ( WorkingDialogManager * ) clientData;

	obj->timer();
}


void WorkingDialogManager::timer()
{
	if ( !_w )
		return;
	
	setTimeout();

	_intervalId = XtAppAddTimeOut ( XtWidgetToApplicationContext ( _w ),
									_timerInterval, 
									&WorkingDialogManager::timerCallback,
									( XtPointer ) this );

	XtVaSetValues ( _w, XmNsymbolPixmap, _busyPixmaps->next(), NULL );
}


void WorkingDialogManager::unpostCallback ( Widget, XtPointer clientData, XtPointer )
{
	WorkingDialogManager *obj = ( WorkingDialogManager * ) clientData;
	obj->unpost();
}


void WorkingDialogManager::unpost()
{
	assert ( _w );
	XtUnmanageChild ( _w );	// remove dialog from screen
		
	if ( _intervalId )
		XtRemoveTimeOut ( _intervalId );	// stop animation

	while ( _busyPixmaps->currentPixmap() != _busyPixmaps->numPixmaps() )
		_busyPixmaps->next();	

	_updateInterval = 0;
}


void WorkingDialogManager::updateMessage ( char *text )
{
	if ( _w )
	{
	    strncpy( _text, text, sizeof ( _text ) );
	    _text [ sizeof ( _text ) - 1 ] = 0;

	    XmString xmstr = XmStringCreateSimple ( text );
	    XtVaSetValues ( _w, XmNmessageString, xmstr, NULL );
	    XmStringFree ( xmstr );
	}
}


void WorkingDialogManager::updateTimeout ( unsigned long progressInterval )
{
	if ( progressInterval >= 0 )
		_updateInterval = progressInterval;
}


void WorkingDialogManager::setTimeout()
{	
	if ( _updateInterval > 0 )
	{
		unsigned int progressValue = 0;

		// if somewhere in chars 1 to 3 is a"%", presume this is a text percentageDone
		// if so, take advantage of the fact to update _updateInterval if necessary

		for ( int i = 1; i < 4; i++)
		{
			if ( _text[i] == '%' )
	 			progressValue = 1;
		}

		if ( progressValue == 1 )
		{
			progressValue =  atoi ( ( char * ) _text );

			progressValue /= ( _busyPixmaps->numPixmaps() - 1 );

			if ( progressValue > ( _busyPixmaps->currentPixmap() - 1 ) )	// % ahead of bar
				_updateInterval /= 2;

			if ( progressValue < ( _busyPixmaps->currentPixmap() - 1 ) )
			{
				_busyPixmaps->previous();	// keep the existing pixmap
				_updateInterval *= 2;
			}
		}
		
		_timerInterval = _updateInterval;
	}
	else
		_timerInterval = 200;	// default pixmap update every 200 mS
}

